@@ -512,7 +512,7 @@ bool JCCEnv::restorePythonException(jthrowable throwable) const
     {
         jfieldID fid = vm_env->GetFieldID(pycls, "py_error_state", "J");
         PyObject *state = (PyObject *) vm_env->GetLongField(throwable, fid);
-        
+
         if (state != NULL)
         {
             PyObject *type = PyTuple_GET_ITEM(state, 0);
@@ -955,6 +955,7 @@ jstring JCCEnv::fromUTF32(const uint32_t *chars, jsize len) const
 {
     std::vector<jchar> jchars;
 
+    jchars.reserve((size_t) len);
     for (jsize i = 0; i < len; ++i) {
         uint32_t c = chars[i];
 
@@ -1004,9 +1005,17 @@ jstring JCCEnv::fromPyString(PyObject *object) const
               }
           }
 
-          case PyUnicode_1BYTE_KIND:
-              return get_vm_env()->NewStringUTF(
-                  (const char *) PyUnicode_1BYTE_DATA(object));
+          case PyUnicode_1BYTE_KIND: {
+              Py_ssize_t len = PyUnicode_GET_LENGTH(object);
+              Py_UCS1 *pchars = PyUnicode_1BYTE_DATA(object);
+              std::vector<jchar> jchars((size_t) len);
+
+              for (std::vector<jchar>::iterator it = jchars.begin();
+                   it != jchars.end(); ++it)
+                  *it = (jchar) *(pchars++);
+
+              return get_vm_env()->NewString(jchars.data(), (size_t) len);
+          }
 
           case PyUnicode_2BYTE_KIND: {
               Py_ssize_t len = PyUnicode_GET_LENGTH(object);
